 PROGRAM Dial;
(* Author      :  T. Ron Davis
   Date        :  Sep/84
   Version     :  2.0
   Description :  DIAL is a dialing program that allows a user
                  to keep track of frequently called systems,
                  their associated access codes, passwords, etc.
                  It is currently set up to display a list of
                  up to 20 systems and autodial via a DEC DF03-AC
                  modem.  This particular version uses several
                  ANSI escape sequences unique to the DEC Rainbow,
                  (bold, blink, reverse video) but the program can
                  be easily adapted for any system.  The Modem (DF03)
                  commands can also be adapted for other autodial
                  modems.  For additional information, please refer
                  to DIAL.DOC.  This program is donated "as is" to
                  the public domain as a small thank you for the many
                  applications that I have acquired via this
                  source over the past years.  The orginal author
                  information should be maintained as this
                  application is modified and enhanced upon.

                  Thanks to Jim Dowling's XEQCMD.PAS procedure,
                  Version 2.0 added the capability to execute
                  the user's requested terminal emulation program.
                  Currently, DIAL is setup to activate KERMIT
                  when the connection is established *)

 (* NOTE -  DIAL uses data contained in the ASC file DIALNUM.DOC *)

TYPE
  VT100Str = String[4];

  String127 = String[127];    { CCP command string }

  Register_Set = Record       { Machine registers  }
       AX, BX, CX, DX, BP, SI, DI, DS, ES, FLAGS : Integer;
  End;

CONST
    AtoT : Set of Char = ['A'..'T','a'..'t'];
    Off   = 0;
    Bold  = 1;
    Blink = 5;
    Rev   = 7;

VAR
  ch,modresp: Char;
  i,sel:      Integer;
  thefil:     Text;
  inputline:  String[80];
  commandline: String127;
  items:      Array[1..20] of String[80];
  letsquit:   Boolean;

FUNCTION GetChr: Char;
  CONST
    AtoT : Set of Char = ['A'..'T','a'..'t'];
  VAR
    C  : Char;
    OK : Boolean;
  BEGIN
    REPEAT
      Read(Kbd, C);
      OK := (C in AtoT);
      If Not OK then Write(Chr(7));
    UNTIL OK;
    GetChr := C;
  END; (* Function GetChr *)

FUNCTION VT100(Func: Integer): VT100Str;
(*  Usage is  VT100(n)   where n is the requested attribute -
    The following VT100 attributes are supported:
         0 - All attributes off
         1 - Bold On
         4 - Underscore
         5 - Blink On
         7 - Reverse Video                  *)
VAR
  str1: String[1];
  ch:   char;
BEGIN
  Str(Func,str1);
  ch := str1;
  VT100 := Chr($1B)+'['+ch+'m';
END (* function VT100 *);

FUNCTION ASC(ch1: char): integer;
(* Function is identical to the ASC function in BASIC -
   it converts a single character into it's ASC decimal
   equivilent -  i.e.  an  'A' would return as a 65.
   usage is    asc(ch)   where ch is a single character *)
var
  temp1,temp2: integer;
begin
  val(ch1,temp1,temp2);
  asc := temp1+48;
end (* function ASC *);

PROCEDURE PaintScreen;
BEGIN
   Clrscr;
   Crtexit;
   i := 1;
   Write(VT100(Bold),'#   Telephone     Description      Account');
   Writeln('        Password          Misc.',VT100(Off));
   Write(' +************************************************');
   Writeln('****************************+ ');
   Assign(thefil,'DIALNUM.DOC');
   Reset(thefil);
   Readln(thefil,inputline);
   Readln(thefil,inputline);
   REPEAT
   BEGIN
       Readln(thefil,inputline);
       ch := COPY(inputline,1,1);
       items[i] := inputline;
       If ch <> '*' THEN Writeln(inputline);
       i := i + 1;
   END;
   UNTIL not (ch in AtoT);
   Write(' +************************************************');
   Writeln('****************************+ ');
 END;

PROCEDURE GetSel;
BEGIN
   Gotoxy(25,24);
   Write(VT100(Bold),'     Enter your selection - ');
   ch := UpCase(GetChr);
   Write(ch,VT100(Off));
   i := ord(ch) - 64;
   Gotoxy(1,i+2);
   Write(VT100(Rev),items[i],VT100(Off));
END;

PROCEDURE InitModem;
(* This procedure will turn on DTR and issue two ^Bs to
   a DFO3 Modem on the Rainbow Communication port *)
begin
  PORT[2] := 138;
  delay(500);
  PORT[64] := 2;
  delay(1000);
  PORT[64] := 2;
  delay(1000);
END (* procedure INITMODEM *);

PROCEDURE DialIt;
CONST
one2nine : Set of Char = ['0'..'9'];

VAR
ndx: integer;
thenum: String[14];
BEGIN
  thenum := COPY(items[i],3,15);
  ndx := 1;
  Gotoxy(3,i+2);
  Write(thenum);
  Write(VT100(Bold));
  Repeat
   BEGIN
     IF (thenum[ndx] in one2nine) THEN
       BEGIN
         PORT[64] := ASC(thenum[ndx]);
         Gotoxy(ndx+2,i+2);
         Write(thenum[ndx]);
         delay(1350);
       END;
     ndx := ndx+1;
   END;
  Until thenum[ndx] = '|';
  Write(VT100(Off));
END (* procedure dialit *);

PROCEDURE GetModem;
CONST
validresp: Set of Char = ['A','B'];

VAR
secs: integer;
BEGIN
(* 15 seconds delay for a direct call - 45 seconds
   delay for operator intervention *)

  IF COPY(items[i],3,1) = '0' THEN secs := 45
  ELSE secs := 15;

  REPEAT
    BEGIN
      modresp := chr(PORT[64]);
      Gotoxy(76,24);
      ClrEol;
      Write(modresp,' ',secs);
      If modresp = 'A' THEN secs := 1;
      Delay(1000);
      secs := secs-1;
    END;
  UNTIL secs = 0;
  IF modresp = 'A' THEN
   BEGIN
     Gotoxy(25,24);
     ClrEol;
     Write(VT100(Blink),'      Connection Established',chr(7),chr(7));
     Write(VT100(Off));
   END;
  IF modresp = 'B' THEN
   BEGIN
     Gotoxy(25,24);
     ClrEol;
     Write (VT100(Blink),'     No Answer or Line Busy',chr(7),chr(7));
     Write(VT100(Off));
     Delay(1500);
   END;
  IF not (modresp in validresp) THEN
   BEGIN
     Gotoxy(25,24);
     ClrEol;
     Write(VT100(Blink),' Inoperable Modem Interface',chr(7),chr(7));
     Write(VT100(Off));
     Delay(1500);
   END;
END (* Procedure GetModem *);

Procedure Execute_Command ( command_line : String127 );
{ Author: Jim Dowling    Creation Date: 12-Jul-1984 }

Const
   NUL    = #$00;              { Command termination chartacter        }
   SetOff = $1A;               { Set DMA offset address function code  }
   Xeqcml = $2F;               { Execute command line function code    }
   SetSeg = $33;               { Set DMA segment address function code }
   GetSeg = $34;               { Get DMA segment address function code }

Var
   Register  : Register_Set;   { 8086/8088 register definitions }
   i         : Integer;

Begin
   { Capitalize the command string }
   For i := 1 To Length(command_Line) Do
       command_line[i] := Upcase(command_line[i]);

   { Terminate the command line }
   command_line := command_line + NUL;

   { Set the command buffer offset }
   Register.CX := SetOff;
   Register.DX := Ofs(Command_Line[1]);
   Bdos(Register);

   { Set the command buffer segment }
   Register.CX := SetSeg;
   Register.DX := Seg(Command_Line[1]);
   Bdos(Register);

   { Call BDOS to execute it }
   Register.CX := Xeqcml;
   Bdos(Register);
End (* Execute_Command *);


BEGIN (* Main Program *);
  PaintScreen;
  GetSel;
  InitModem;
  DialIt;
  GetModem;
  IF modresp = 'A' THEN
    BEGIN
       ClrScr;
       Gotoxy(1,24);
       Writeln(chr(7),chr(7),'Activating KERMIT .....');
       Writeln(VT100(Rev),items[i],VT100(Off));
       commandline := 'KERMIT';
       Execute_Command(commandline);
    END;

END.

