procedure swaplevel(i:integer);
begin
  global.swaplevel:=i;
end;

procedure setchain(name:fullid);
begin
  global.chainid:=name;
end;

procedure setcval(s:string);
begin
  global.cval:=s;
end;

procedure getcval(var s:string);
begin
  s:=global.cval;
end;

procedure getfinfo(var f:fib; var i:direntry);
begin
  i:=f.fheader;
end;

procedure setfinfo(var f:fib; i:direntry);
begin
  f.fheader:=i;
end;

{ return true if the file is open }
function fileopen(f:fib):boolean;
begin
  fileopen:=f.fisopen;
end;

{ set the type of file to required type }
procedure settype(var f:fib; t:filekind);
begin
  f.fmodified:=true;
  f.fheader.dfkind:=t;
end;

{ set the date of a file }
procedure setfdate(var f:fib; d:daterec);
begin
  f.fmodified:=true;
  with f.fheader do
    case dfkind of
      securedir,untypedfile: dlastboot:=d;
    otherwise
      daccess:=d;
    end;        { case }
end;

procedure getvol(f:fib; var volid:vid);
begin
  volid:=f.fvid;
end;

procedure getvnum(f:fib; var i:integer);
begin
  i:=f.funit;
end;


procedure getfullid(f:fib; var s:string);
begin
  with f do
    s:=concat(fvid,':',fheader.dtid);
end;

function filesize(f:fib):integer;
begin
  if f.fmaxblk>(f.fheader.dlastblk-f.fheader.dfirstblk) then
    filesize:=f.fmaxblk
  else
    filesize:=(f.fheader.dlastblk-f.fheader.dfirstblk);
end;

procedure getsameid(var vol:vid);
begin
  vol:=concat(global.userinfo.codefibp^.fvid,':');
end;

procedure getcodename(var id:fullid);
begin
  id:=concat(global.userinfo.codefibp^.fvid,':',
             global.userinfo.codefibp^.fheader.dtid);
end;

PROCEDURE getcommand( VAR command:char; commandset:charset );
{ get a single character ignoring case, limited to commandset }
BEGIN
   REPEAT
      command:=chr(0);
      unitread(2,command,1,0,0);
      if command in ['a'..'z'] then
        command:=chr(ord(command)-ord('a')+ord('A'))
   UNTIL command IN commandset;
END; { getcommand }

PROCEDURE getachar( VAR ch:char; legalset:charset ); 
{return single character from keyboard}
BEGIN
   REPEAT
      ch:=chr(0);
      unitread(2,ch,1,0,0);
   UNTIL (ch in legalset)
END; {getachar}


PROCEDURE shiftup( VAR ch:char );
{ convert a lower case character to upper case }
BEGIN
   IF ch IN [ 'a'..'z' ] THEN
     ch := chr( ord( ch ) -ord('a')+ord('A'))
END; { shiftup }



PROCEDURE bumpstrup( VAR s:string );
{shift characters in string to upper case}
VAR
   i    : integer;
   ch   : char;
BEGIN
   IF length(s) > 0 THEN
   FOR i:=1 TO length(s) DO
      if s[i] in ['a'..'z'] then
        s[i]:=chr(ord(s[i])-ord('a')+ord('A'))
END; {bumpstrup}


FUNCTION inrange( n, min, max : integer ) : boolean  ;
{ true if min <= n <= max }
BEGIN
   inrange := ( min <= n ) AND ( n <= max )
END; { inrange }


FUNCTION verified( message:string ):boolean  ;
{true if 'Y' or 'y' typed in response to message}
VAR ch: char;
     s: string;
BEGIN
   unitwrite(2,message[1],ord(message[0]),0,52);
   s:=' ? (Y/N) ';
   unitwrite(2,s[1],ord(s[0]),0,52);
   ch:=chr(ord('Z')-ord('@'));  { a control-z to  "CLEOLN" }
   unitclear(2);
   repeat
     ch:=chr(0);
     unitread(2,ch,1,0,0);
   until ch in ['y','Y','n','N'];
   unitwrite(1,ch,1,0,52);
   verified:=ch in ['Y','y'];
END;  {verified}


FUNCTION val( ch:char ):integer ;
{returns integer value of a character}
BEGIN
   if ch in ['0'..'9'] then
     val := ord(ch) - ord('0')
   else
     val:=-1;
END; {val}


FUNCTION value(VAR s:string ):integer ;
{returns integer value of a string}
VAR
   a:       real;
   i, z:    integer;
   ch:      char;
   neg:     boolean;
   gs:      string;
BEGIN
   i := 1;
   z := ord('0');
   a := 0.0;
   gs := concat( s, '/' );
   WHILE gs[1] = ' ' DO
     delete( gs, 1, 1 );
   ch := gs[ i ];
   IF ch = '-' THEN
      BEGIN
         neg := true;
         i := i + 1;
         ch := gs[ i ]
      END
   ELSE
      BEGIN
         neg := false;
         IF ch = '+' THEN
            BEGIN
               i := i + 1;
               ch := gs[ i ]
            END
      END;
   WHILE ch IN [ '0'..'9' ] DO
      BEGIN
         a := 10 * a + ord(ch) - z;
         i := i + 1; ch := gs[ i ]
      END;
   IF neg THEN a := -a;
   IF (a>= -32768.0) AND (a<=32767.0)
      THEN  value := trunc(a)
      ELSE  value := 0
END;  {value}


FUNCTION rvalue( VAR s:string ):real ;
{real value of a string with a decimal part}
VAR
   i, z:     integer;
   a, f:     real;
   ch:       char;
   neg:      boolean;
   gs:      string;
BEGIN
   i := 1; z := ord('0'); a := 0; gs := concat( s, '/' );
   WHILE gs[1] = ' ' DO delete( gs, 1, 1 );
   ch := gs[ i ];
   IF ch = '-' THEN
      BEGIN
         neg := true; i := i + 1; ch := gs[ i ]
      END
   ELSE
      BEGIN
         neg := false;
         IF ch = '+' THEN
            BEGIN
               i := i + 1; ch := gs[ i ]
            END
      END;
   IF ch IN [ '0'..'9' ]
   THEN
      REPEAT {get integer part}
         a := 10 * a + ord(ch) - z;
         i := i + 1; ch := gs[ i ]
      UNTIL NOT ( ch IN [ '0'..'9' ] );
   IF ch = '.' THEN
      BEGIN f := 0.1;
         i := i + 1; ch := gs[ i ];
         WHILE ch IN [ '0'..'9' ] DO
            BEGIN
               a := a + ( ord(ch) - z ) * f;
               f := f * 0.1;
               i := i + 1; ch := gs[ i ]
            END
      END;
   IF neg THEN a := -a; rvalue := a
END; {rvalue}


{
for encrypt and decrypt, if you don't have bytestream, use STRING and
declare it in your program procedure xxx( var data; var seed:string; etc...
}
{ encrypt data and seed using 'A' algorithm }
procedure encrypta(var data:bytestream;var seed:string; datalen:integer);
var
  i,j:integer;
  buf:integer;
  eor,eor1:record case boolean of
            true:(ch:char);
            false:(b:boolean);
           end;
  
begin
  j:=1;
  for i:=0 to datalen-1 do
    begin
      eor.ch:=data[i];
      eor1.ch:=seed[j];
      { exclusive-or data and seed }
      eor.b:=(not((eor.b and eor1.b) or ((not eor.b)and(not eor1.b))));
      data[i]:=eor.ch;
      buf:=ord(eor.ch)+ord(eor1.ch);
      j:=j+1;
      if j>ord(seed[0]) then
        j:=1;
      buf:=buf+ord(seed[j]);
      j:=j-1;
      if j<1 then
        j:=ord(seed[0]);
      seed[j]:=chr(buf mod 256);
      j:=j+1;
      if j>ord(seed[0]) then
        j:=1;
    end;
end;

{ decrypt data and seed using 'A' algorithm }
procedure decrypta(var data:bytestream;var seed:string; datalen:integer);
var
  i,j:integer;
  buf:integer;
  eor,eor1:record case boolean of
            true:(ch:char);
            false:(b:boolean);
           end;
  
begin
  j:=1;
  for i:=0 to datalen-1 do
    begin
      eor.ch:=data[i];
      eor1.ch:=seed[j];
      { exclusive-or data and seed }
      eor1.b:=(not((eor.b and eor1.b) or ((not eor.b)and(not eor1.b))));
      data[i]:=eor1.ch;
      buf:=ord(seed[j])+ord(eor.ch);
      j:=j+1;
      if j>ord(seed[0]) then
        j:=1;
      buf:=buf+ord(seed[j]);
      j:=j-1;
      if j<1 then
        j:=ord(seed[0]);
      seed[j]:=chr(buf mod 256);
      j:=j+1;
      if j>ord(seed[0]) then
        j:=1;
    end;
end;

{ restart a program from scratch }
procedure restart;
begin
  with global.userinfo.codefibp^ do
    global.chainid:=concat(fvid,':',fheader.dtid);
  exit(pcode);
end;


{ find the PASCAL.AREA address on a large disk }
function pascal_area(unitnum:integer):integer;

var
  buffer:packed array[0..2047] of char;
  i,temp:integer;
  

begin
  temp:=0;
  unitread(unitnum,buffer,sizeof(buffer),2);
  for i:=0 to 2030 do
    begin
      if buffer[i]='P' then
        if buffer[i+1]='A' then
          if buffer[i+2]='S' then
            if buffer[i+3]='C' then
              if buffer[i+4]='A' then
                if buffer[i+5]='L' then
                  if buffer[i+6]='.' then
                    if buffer[i+7]='A' then
                      if buffer[i+8]='R' then
                        if buffer[i+9]='E' then
                          if buffer[i+10]='A' then
                            begin
                              temp:=ord(buffer[i+16])+(256*ord(buffer[i+17]));
                              i:=2047;
                            end;
    end;
  pascal_area:=temp;
end;


procedure vdirinfo(unitnum,blocknum,offset:integer;
                   var start,vlength:integer;
                   var name:string);

  type
    rtype = (header, regular);
  
  var
    startblk:integer;
    vdir : record case boolean of
              true:(blk:packed array[0..1023] of char);
              false:(dir:record
                           vdir1:array[0..31] of
                                   packed record case rtype of
                                     header:(pseudo_device_length:integer;
                                             cur_num_vols:integer;
                                             ppmname:string[3]);
                                     regular:(start:integer;
                                              length:integer;
                                              default_unit:0..255;
                                              filler:0..127;
                                              wp:boolean;
                                              olddriveaddr:integer)
                                    end;
                           vdesc:array[0..31] of string[15];
                           vnames:array[0..31] of string[7]
                         end )
          end;
                   

  begin
    name:='';
    start:=0;
    vlength:=0;
    unitread(unitnum,vdir.blk,1024,blocknum);
    if ioresult=0 then
    with vdir.dir do
      if offset = 0 then
        begin
          name:=copy(vdir1[offset].ppmname,1,3);
          start:=vdir1[offset].cur_num_vols;
          vlength:=vdir1[offset].pseudo_device_length;
        end
      else
        begin
          name:=copy(vnames[offset],1,length(vnames[offset]));
          vlength:=vdir1[offset].length;
          start:=vdir1[offset].start;
        end;
  end;
  
function csum:integer;
var a:record case boolean of
      true:(i:integer);
      false:(p:^string);
    end;
  b:integer;
begin
  a.i:=1;
  with a do
    while i<256 do
      begin
        b:=(b+ord(p^[0])) mod 32000;
        i:=i+1;
      end;
  csum:=b;
end;

function csum2(var data:bytestream; length:integer):integer;
var
  i,j:integer;
begin
  j:=0;
  for i:=0 to length do
    j:=(j+ord(data[i])) mod 32000;
  csum2:=j;
end;

{ calculate the sine of a number in radians }
function sine(x:real):real;
var
  isnegative:boolean;
  r:real;
  i:integer;
begin
  isnegative:=(x<0.0);
  x:=abs(x);
  if x>32767.0 then
    begin
      x:=1.0e30;
      x:=x*x;
    end;
  i:=round(x*0.318309903144);
  if odd(i) then
    isnegative:=not(isnegative);
  x:=(x-(i*3.140625))-(i*0.0009676536560058);
  if abs(x)>0.00025 then
    begin
      r:=x*x;
      x:=x+(x*(((((((0.000002601902461*r)-0.0001980742073)*r)+
         0.00833302402496)*r)-0.16666656494)*r));
    end;
  if isnegative then
    x:=-x;
  sine:=x;
end;

function cosine(x:real):real;
var
  isnegative:boolean;
  r:real;
  i:integer;
begin
  isnegative:=false;
  x:=abs(x)+1.570796327;
  if x>32767.0 then
    begin
      x:=1.0e30;
      x:=x*x;
    end;
  i:=round(x*0.318309903144);
  if odd(i) then
    isnegative:=not(isnegative);
  x:=(x-(i*3.140625))-(i*0.0009676536560058);
  if abs(x)>0.00025 then
    begin
      r:=x*x;
      x:=x+(x*(((((((0.000002601902461*r)-0.0001980742073)*r)+
         0.00833302402496)*r)-0.16666656494)*r));
    end;
  if isnegative then
    x:=-x;
  cosine:=x;
end;



function tangant(x:real):real;
var
  isnegative:boolean;
  temp,r,sin,cos:real;
  i:integer;
begin
  isnegative:=(x<0.0);
  temp:=abs(x);
  if temp>32767.0 then
    begin
      temp:=1.0e30;
      temp:=temp*temp;
    end;
  i:=round(temp*0.318309903144);
  if odd(i) then
    isnegative:=not(isnegative);
  temp:=(temp-(i*3.140625))-(i*0.0009676536560058);
  if abs(temp)>0.00025 then
    begin
      r:=temp*temp;
      temp:=temp+(temp*(((((((0.000002601902461*r)-0.0001980742073)*r)+
         0.00833302402496)*r)-0.16666656494)*r));
    end;
  if isnegative then
    temp:=-temp;
  sin:=temp;
  isnegative:=false;
  temp:=abs(x)+1.570796327;
  if temp>32767.0 then
    begin
      temp:=1.0e30;
      temp:=temp*temp;
    end;
  i:=round(temp*0.318309903144);
  if odd(i) then
    isnegative:=not(isnegative);
  temp:=(temp-(i*3.140625))-(i*0.0009676536560058);
  if abs(temp)>0.00025 then
    begin
      r:=temp*temp;
      temp:=temp+(temp*(((((((0.000002601902461*r)-0.0001980742073)*r)+
         0.00833302402496)*r)-0.16666656494)*r));
    end;
  if isnegative then
    temp:=-temp;
  cos:=temp;
  tangant:=sin/cos;
end;

function readtsect(unitnum, offset, tracknum, sectnum:integer;
                   var buffer:wordstream):integer;
VAR
 i,blocknum,temp,size:INTEGER;
 block:packed array[0..511] of 0..255;
BEGIN
  unitstatus(unitnum,size,1);
  if size>=400 then
    size:=2
  else
    size:=1;
  fillchar(block,sizeof(block),0);
  (* compute half-block corresponding to desired sector  *)
  blocknum:=(sectnum div 8)*16;
  sectnum:=sectnum mod 8;
  IF (sectnum IN [0,15]) THEN
    blocknum:=sectnum+blocknum
  ELSE
    blocknum:=(15-sectnum)+blocknum;
  
  IF (odd(blocknum)) THEN
    temp:=256
  ELSE
    temp:=0;
  
  (* now compte blocknum off set from track 0 *)
  blocknum:=(blocknum DIV 2)+(8*tracknum*size);
  unitread(unitnum,block,sizeof(block),offset*800+blocknum);
  blocknum:=ioresult;
  if blocknum=0 then
    for i:=0 to 255 do
      buffer[i]:=block[temp+i];
  readtsect:=blocknum;
END;


function writetsect(unitnum, offset, tracknum, sectnum:integer;
                   var buffer:wordstream):integer;
 VAR
  ok:boolean;
  error,i,blocknum,temp,size:INTEGER;
  block:packed array[0..511] of 0..255;
 BEGIN
   unitstatus(unitnum,size,1);
   if size>=400 then
     size:=2
   else
     size:=1;
   fillchar(block,sizeof(block),0);
   (* compute half-block corresponding to desired sector  *)
   blocknum:=(sectnum div 8)*16;
   sectnum:=sectnum mod 8;
   IF (sectnum IN [0,15]) THEN
     blocknum:=sectnum+blocknum
   ELSE
     blocknum:=(15-sectnum)+blocknum;
   
   IF (odd(blocknum)) THEN
     temp:=256
   ELSE
     temp:=0;
   
   (* now compte blocknum off set from track 0 *)
   blocknum:=(blocknum DIV 2)+(8*tracknum*size)+(800*offset);
   unitread(unitnum,block,sizeof(block),blocknum);
   error:=ioresult;
   if error=0 then
     begin
       for i:=0 to 255 do
         block[temp+i]:=buffer[i];
       unitwrite(unitnum,block,sizeof(block),blocknum);
       error:=ioresult;
     end;
   writetsect:=blocknum;
 END;

function VTOC_READ(unitnum:integer; var VTOC:VTOC_structure;
                   offset:integer):boolean;

var
  ok:boolean;
  size:integer;
  buffer:packed array[0..1] of VTOC_structure;

Begin  (* MAIN PROGRAM  *)
  ok:=true;
  unitstatus(unitnum,size,1);
  if size>=400 then
    size:=2
  else
    size:=1;
  Unitread (unitnum,buffer,((136*size)+(offset*800)),512);
  ok:=ioresult=0;
  if ok then
    VTOC:=buffer[0];
  VTOC_read:=ok;
End;


procedure get_sys_info(runtime:boolean; p_sys_size:integer;
                       p_sys_ver:string);
  type
    pab = packed array[0..0] of bytes;
    pab1 = packed array[0..7] of boolean;
  var
    i:integer;
  TRIX1:   RECORD CASE BOOLEAN OF
              TRUE: (ADDRESS:INTEGER);
              FALSE:(POINTER:^PAB);
            END;
  TRIX2:   RECORD CASE BOOLEAN OF
              TRUE: (INT:bytes);
              FALSE:(BIN:PAB1);
            END;

  begin
    trix1.address:=-16606;
    trix2.int:=trix1.pointer^[0];
    trix1.address:=-16607;
    case trix1.pointer^[0] of
      2: p_sys_ver := '1.1';
      3: p_sys_ver := '1.2';
      4: p_sys_ver := '1.3';
      otherwise
         p_sys_ver := 'unk';
    end;
    runtime:=trix2.bin[0];
    with trix2 do
      begin
        bin[1]:=bin[5];
        bin[0]:=bin[6];
        for i:=2 to 7 do
          bin[i]:=false;
        case int of
          0:p_sys_size:=64;
          1:p_sys_size:=128;
          2:p_sys_size:=48;
          3:p_sys_size:=0;
        end;
      end;
  end;


function findf(var tytle:string):boolean;
label 1;
var
  found:boolean;
  phyle:file;
  
  begin                         { findf }
    reset(phyle,tytle);
    if ioresult<>0 then
      begin
        reset(phyle,concat(tytle,'.code'));
        if ioresult<>0 then
          begin
            reset(phyle,concat(tytle,'.text'));
            if ioresult<>0 then
              begin
                reset(phyle,concat(tytle,'.info'));
                if ioresult<>0 then
                  begin
                    reset(phyle,concat(tytle,'.data'));
                    if ioresult<>0 then
                      begin
                        reset(phyle,concat(tytle,'.graf'));
                        if ioresult<>0 then
                          begin
                            reset(phyle,concat(tytle,'.foto'));
                            if ioresult<>0 then
                              begin
                                reset(phyle,concat(tytle,'.bad'));
                                if ioresult<>0 then
                                  begin
                                    found:=false;
                                    goto 1;
                                  end
                                else
                                  begin
                                    tytle:=concat(tytle,'.bad');
                                    found:=true;
                                    goto 1;
                                  end;
                              end
                            else
                              tytle:=concat(tytle,'.foto');
                          end
                        else
                          tytle:=concat(tytle,'.graf');
                      end
                    else
                      tytle:=concat(tytle,'.data');
                  end
                else
                  tytle:=concat(tytle,'.info');
              end
            else
              tytle:=concat(tytle,'.text');
          end
        else
          tytle:=concat(tytle,'.code');
      end;
  1:    findf:=found;
  end;                          { findf }



