program ProcParamTest;
{$C (C) Copyright 1988, Pecan Software Systems, Inc.}
{
 Author: Dr. Gerald Weiss, Pecan Software Systems, Inc.
 
 This program illustrates a technique for faking procedure variables
 (as supported in other Power System languages, like Modula-2 and C)
 in UCSD Pascal.
 
 The critical procedures here are passign, which assigns the procedure
 to the variable of choice -- any variable of type ProcType; in this
 case, an element of array MyProcs, and pcall, which calls the contents
 of the specified procedure variable.
 
 Limits: The procedure passign is specific to the type of the procedure,
 so every procedure treated as a different type as defined by Pascal
 (for example, different number or type of parameters) must have its own
 variant of passign.
 
 Also, note that handling of parameter passing and function return values
 is entirely up to the programmer.
 
 In short, know what you are doing when you use these routines!
}
const
  LDM = 208; {load multiple p-code}
  CPF = 151; {call formal procedure p-code}
type 
  ProcType = array[1..3] of integer;  {the p-machine stores procedures as}
var				      {3-word values}
  MyProcs: array[1..5] of ProcType;   {an array of procedures (arbitrary size}
  i: integer;
 
procedure p1; begin writeln('In p1'); end; {some sample procedures}
procedure p2; begin writeln('In p2'); end;
procedure p3; begin writeln('In p3'); end;
procedure p4; begin writeln('In p4'); end;
procedure p5; begin writeln('In p5'); end;
 
procedure passign(procedure p; dummy: integer; var a: ProcType);
var 
  t: integer;
  pt: ^integer;
begin
   pmachine(^t, ^dummy, :=);
   t := t + 2;
   pmachine(^pt, (t), :=);
   moveleft(pt^, a, 6);
end;
 
procedure pcall(var p: ProcType);
begin
  pmachine(^p, LDM, 3);
  pmachine(CPF);
end;
 
begin {ProcParamTest}
  passign(p1, 0, MyProcs[1]);
  passign(p2, 0, MyProcs[2]);
  passign(p3, 0, MyProcs[3]);
  passign(p4, 0, MyProcs[4]);
  passign(p5, 0, MyProcs[5]);
  
  for i := 1 to 5 do
    pcall(MyProcs[i]);
end.
 
